﻿using System.Collections.Generic;

namespace HeuristicLab.Algorithms.DataAnalysis.SymRegGrammarEnumeration {

  public abstract class Symbol {
    public string StringRepresentation { get; }

    protected Symbol(string representation) {
      StringRepresentation = representation;
    }

    public override string ToString() {
      return StringRepresentation;
    }
  }

  public class TerminalSymbol : Symbol {
    public readonly bool IsVariable;

    public TerminalSymbol(string representation, bool isVariable = false) : base(representation) {
      IsVariable = isVariable;
    }
  }

  public class NonterminalSymbol : Symbol {
    public List<Production> Alternatives { get; }

    public NonterminalSymbol(string representation) : base(representation) {
      Alternatives = new List<Production>();
    }

    public void AddProduction(params Symbol[] production) {
      Alternatives.Add(new Production(production));
    }
  }

  public class VariableSymbol : NonterminalSymbol { // Convenience class
    public IEnumerable<TerminalSymbol> VariableTerminalSymbols { get; }

    public VariableSymbol(string representation, IEnumerable<string> variableNames) : base(representation) {
      List<TerminalSymbol> createdSymbols = new List<TerminalSymbol>();
      VariableTerminalSymbols = createdSymbols;

      foreach (string variableName in variableNames) {
        TerminalSymbol s = new TerminalSymbol(variableName, isVariable: true);
        createdSymbols.Add(s);
        AddProduction(s);
      }
    }
  }

  public class Production : List<Symbol> {

    public Production(params Symbol[] symbols) : base(symbols) { }

    public Production(IEnumerable<Symbol> symbols) : base(symbols) { }

    public override string ToString() {
      return string.Join(" ", this);
    }
  }
}
